@@ -1,7 +1,7 @@
 #include "include/rcore.h"
 #include "include/globals.h"
 #include "include/matrix.h"
-#include "include/dataframe.h"
+#include "include/data.frame.h"
 #include "include/graph.h"
 
 SEXP smart_network_averaging(SEXP arcs, SEXP nodes, SEXP weights) {
