@@ -1,27 +1,27 @@
 #include "include/rcore.h"
 #include "include/sets.h"
-#include "include/tests.h"
 #include "include/data.frame.h"
+#include "include/contingency.tables.h"
 
 double dlik(SEXP x, double *nparams) {
 
-int i = 0;
-int *n = NULL, *xx = INTEGER(x), llx = NLEVELS(x), num = length(x);
 double res = 0;
+counts1d marginal = { 0 };
 
   /* initialize the contingency table. */
-  fill_1d_table(xx, &n, llx, num);
+  marginal = new_1d_table(NLEVELS(x));
+  fill_1d_table(INTEGER(x), &marginal, length(x));
 
   /* compute the entropy from the marginal frequencies. */
-  for (i = 0; i < llx; i++)
-    if (n[i] != 0)
-      res += (double)n[i] * log((double)n[i] / num);
+  for (int i = 0; i < marginal.llx; i++)
+    if (marginal.n[i] != 0)
+      res += (double)marginal.n[i] * log((double)marginal.n[i] / marginal.nobs);
 
   /* we may want to store the number of parameters. */
   if (nparams)
-    *nparams = llx - 1;
+    *nparams = marginal.llx - 1;
 
-  Free1D(n);
+  Free1DTAB(marginal);
 
   return res;
 
@@ -29,38 +29,25 @@ double res = 0;
 
 double cdlik(SEXP x, SEXP y, double *nparams) {
 
-int i = 0, j = 0, k = 0;
-int **n = NULL, *nj = NULL;
-int llx = NLEVELS(x), lly = NLEVELS(y), num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y);
 double res = 0;
+counts2d joint = { 0 };
 
   /* initialize the contingency table and the marginal frequencies. */
-  n = (int **) Calloc2D(llx, lly, sizeof(int));
-  nj = Calloc1D(lly, sizeof(int));
-
-  /* compute the joint frequency of x and y. */
-  for (k = 0; k < num; k++)
-    n[xx[k] - 1][yy[k] - 1]++;
-
-  /* compute the marginals. */
-  for (i = 0; i < llx; i++)
-    for (j = 0; j < lly; j++)
-      nj[j] += n[i][j];
+  joint = new_2d_table(NLEVELS(x), NLEVELS(y), TRUE);
+  fill_2d_table(INTEGER(x), INTEGER(y), &joint, length(x));
 
   /* compute the conditional entropy from the joint and marginal
        frequencies. */
-  for (i = 0; i < llx; i++)
-    for (j = 0; j < lly; j++)
-      if (n[i][j] != 0)
-        res += (double)n[i][j] * log((double)n[i][j] / (double)nj[j]);
+  for (int i = 0; i < joint.llx; i++)
+    for (int j = 0; j < joint.lly; j++)
+      if (joint.n[i][j] != 0)
+        res += (double)joint.n[i][j] * log((double)joint.n[i][j] / joint.nj[j]);
 
   /* we may want to store the number of parameters. */
   if (nparams)
-    *nparams = (llx - 1) * lly;
+    *nparams = (joint.llx - 1) * joint.lly;
 
-  Free1D(nj);
-  Free2D(n, llx);
+  Free2DTAB(joint);
 
   return res;
 
