@@ -23,6 +23,7 @@ extern SEXP cgpred(SEXP, SEXP, SEXP);
 extern SEXP cgsd(SEXP, SEXP, SEXP);
 extern SEXP chow_liu(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP class_err(SEXP, SEXP);
+extern SEXP colliders(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP configurations(SEXP, SEXP, SEXP);
 extern SEXP count_observed_values(SEXP);
 extern SEXP cpdag(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -53,8 +54,10 @@ extern SEXP is_dag(SEXP, SEXP);
 extern SEXP is_listed(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP is_pdag_acyclic(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP is_row_equal(SEXP, SEXP);
+extern SEXP joint_discretize(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP lw_weights(SEXP, SEXP, SEXP, SEXP);
 extern SEXP mappred(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP marginal_discretize(SEXP, SEXP, SEXP, SEXP);
 extern SEXP match_brace(SEXP, SEXP, SEXP, SEXP);
 extern SEXP mean_strength(SEXP, SEXP, SEXP);
 extern SEXP mi(SEXP, SEXP, SEXP, SEXP);
@@ -87,6 +90,5 @@ extern SEXP tiers(SEXP, SEXP);
 extern SEXP topological_ordering(SEXP, SEXP, SEXP, SEXP);
 extern SEXP tree_directions(SEXP, SEXP, SEXP, SEXP);
 extern SEXP unique_arcs(SEXP, SEXP, SEXP);
-extern SEXP vstructures(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP which_undirected(SEXP, SEXP);
 
