@@ -1,3 +1,4 @@
+#include "data.structures.h"
 
 /* from sampling.c */
 void SampleNoReplace(int k, int n, int *y, int *x);
@@ -7,9 +8,9 @@ void ProbSampleReplace(int n, double *probs, int *values, int ns, int *samples);
 void CondProbSampleReplace(int nprobs, int nconf, double *probs, int *conf,
     int *values, int ns, int *samples, bool *warn);
 
-/* used to be in R core... */
-void c_rcont2(int nrow, int ncol, int *nrowt, int *ncolt, int ntotal,
-    double *fact, int *jwork, int **matrix);
+/* from rcont2.c. */
+void rcounts2d(counts2d table, double *fact, int *workspace);
+void rcounts3d(counts3d table, double *fact, int *workspace);
 
 /* from rbn.c */
 void c_rbn_master(SEXP fitted, SEXP result, SEXP n, SEXP fix, bool debugging);
