@@ -3,7 +3,7 @@
 
 /* independence tests, frontend to be used in R code. */
 SEXP indep_test(SEXP x, SEXP y, SEXP sx, SEXP data, SEXP test, SEXP B,
-    SEXP alpha, SEXP learning) {
+    SEXP alpha, SEXP learning, SEXP complete) {
 
   /* if either node to test is not provided, return a zero-length numeric
    * vector. */
@@ -12,8 +12,8 @@ SEXP indep_test(SEXP x, SEXP y, SEXP sx, SEXP data, SEXP test, SEXP B,
 
   /* filter for NULL and empty strings to make it easy to interface with R. */
   if (length(sx) == 0 || sx == R_NilValue)
-    return utest(x, y, data, test, B, alpha, learning);
+    return utest(x, y, data, test, B, alpha, learning, complete);
   else
-    return ctest(x, y, sx, data, test, B, alpha, learning);
+    return ctest(x, y, sx, data, test, B, alpha, learning, complete);
 
 }/*INDEP_TEST*/
